% Demo script for running R-SVD algorithm an synthetic data
% It generates a random dictionary and an additive noise-affected training
% set with sparse representation, then recovers the generated dictionary
% Jianyi Lin, University of Milan, Italy (jianyi.lin@unimi.it)
% Dec 30, 2015

%% Paths
OMPBOX_PATH = 'ompbox';        %Put OMP-Box v10 compiled in this directory
ROTATESVD_PATH = 'RotateSVD';  %Put RotateSVD in this directory 
addpath(OMPBOX_PATH);
addpath([OMPBOX_PATH '/common']);
addpath(ROTATESVD_PATH);

%% Parameters
n = 50;   % Atom size
m = 100;  % Dictionary size
k = 5;    % Sparsity bound
L = 2000; % Training set size
snr = 10; % Signal to additive noise power
ITERNUM = 100;      % #iterations for R-SVD
s = 10;             % Subdictionary size parameter (# atoms to be taken at each rotation)
SPARSIFIER = 'klimaps'; % 'klimaps' or 'omp' (for k-LiMapS put klimaps.m in the same dir)

normcols = @(arg) arg*spdiags((1./sqrt(sum(arg.*arg)))',0,size(arg,2),size(arg,2));

%% Generate dataset and original dictionary
D = normcols(randn(n,m));
X = zeros(m,L);
for i = 1:L
  p = randperm(m);
  X(p(1:k),i) = randn(k,1);
end
Y = D*X;
Y = normcols(Y) + 10^(-snr/20)*normcols(randn(n,L));

%% Set options
params = struct();
params.data = Y;          % training set
params.Tdata = k;         % sparsity constraint
params.dictsize = m;
params.iternum = ITERNUM;
params.sparsifier = SPARSIFIER;
params.SubdictSize = s;  

%% Launch simulations
[Dnew,Xnew,RMSE] = RotateSVD(params);

%% Measures
Xnorm = norm(X,'fro');
E_SNR = 20*log10(Xnorm./(sqrt(n*L)*RMSE)); % Recons. error expressed in SNR