function Alpha = klimaps(D,Dinv,X,K,max_iter)
% KLIMAPS algorithm
%  ALPHA = KLIMAPS(D,Dinv,X,K) solves the optimization problem
%
%       min  |X - D*ALPHA|_F     s.t.  |Alpha|_0 <= K
%      ALPHA
%
% for each of the signals in X, using klimpas.
% Here, D is a dictionary with normalized columns, Dinv its pseudoinverse,
% X is a matrix containing column signals, K is the # of non-zeros in each
% signal representation, the output ALPHA is a matrix containing the sparse
% representations as its columns.

Alpha = Dinv*X;
m  = size(D,2);
N = size(Alpha,2);
P = eye(m) - Dinv*D;
a = sort(abs(Alpha),'descend');
Lambda = repmat(1./a(K+1,:),m,1);
%Lambda = 1.01*ones(1,N);
for i = 1:max_iter
    %Beta = Alpha.*(1-exp(-Lambda.*abs(Alpha)));
    %Alpha = Beta-Dinv*(D*Beta-X);
    Alpha = Alpha - P*(Alpha.*exp(-Lambda.*abs(Alpha)));
    [a,idx] = sort(abs(Alpha),'descend');
    Lambda = repmat(1./a(K+1,:),m,1);
end
for i = 1:N
    Alpha(idx(K+1:m,i),i) = 0;  % zeroing m-k entries
    Alpha(idx(1:K,i),i) = pinv(D(:,idx(1:K,i)))*X(:,i); % least-squares prj
end
Alpha = sparse(Alpha);

